<?php
/*
Plugin Name: UtopiaPay for WooCommerce
Plugin URI: https://utopiapay.io
Description: Accept payments in Crypton (CRP) and Utopia USD (UUSD) via UtopiaPay.
Version: 1.0.14
Author: UtopiaPay
Author URI: https://utopiapay.io
License: GPLv2 or later
Text Domain: utopiapay
*/

/**
 * UtopiaPay.io Gateway
 * 
 *
 * Provides a UtopiaPay Payment Gateway for WooCommerce.
 *
 * @class 		WC_UtopiaPay
 * @extends		WC_Gateway_UtopiaPay
 * @version		1.0.14
 * @package		WooCommerce/Classes/Payment
 * @author 		UtopiaPay gateway plugin by Teo2Peer
 */



// WordPress plugin to add the UtopiaPay payment gateway to your site
// function to add a menu in the WordPress wp_admin 

add_action('plugins_loaded', 'utopiapay_gateway_load', 0);





function utopiapay_gateway_load()
{


    // check if woocommerce is active else stop the plugin
    if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
        return;
    }


    if (!class_exists('WC_Payment_Gateway')) {
        // oops!
        return;
    }

    function utopiapay_gateway_edd_notice()
    {
        ?>
<div class="error">
    <p><?php _e('UtopiaPay for WooCommerce is not compatible with Easy Digital Downloads enabled beacuse it blocks the IPN callback and do not update payment information', 'utopiapay'); ?>
    </p>
</div>
<?php
    }


    // check if easy-digital-downloads is active
    if (in_array('easy-digital-downloads/easy-digital-downloads.php', apply_filters('active_plugins', get_option('active_plugins')))) {
        // return message that say that utopiapay is not compatible with EDD plugin
        
        add_action('admin_notices', 'utopiapay_gateway_edd_notice');

        return;
    }


    /**
     * Add the gateway to WooCommerce.
     */

    add_filter('woocommerce_payment_gateways', 'add_utopiapay_gateway');


    function add_my_currency( $currencies ) {
        $currencies['uUSD'] = __( 'Utopia USD', 'woocommerce' );
        $currencies['CRP'] = __( 'Crypton', 'woocommerce' );
        return $currencies;
    }
    add_filter( 'woocommerce_currencies', 'add_my_currency' );


    function add_my_currency_symbol( $currency_symbol, $currency ) {
        switch( $currency ) {
            case 'uUSD': $currency_symbol = 'uUSD'; break;
            case 'CRP': $currency_symbol = 'CRP'; break;
        }
        return $currency_symbol;
    }
    add_filter('woocommerce_currency_symbol', 'add_my_currency_symbol', 10, 2);





    function add_utopiapay_gateway($methods)
    {
        if (!in_array('WC_Gateway_Utopiapay', $methods)) {
            $methods[] = 'WC_Gateway_Utopiapay';
        }
        return $methods;
    }

    class WC_Gateway_Utopiapay extends WC_Payment_Gateway
    {
        /**
         * Constructor for the gateway.
         *
         * @access public
         * @return void
         */
        public function __construct()
        {
            global $woocommerce;

            $this->id = 'utopiapay';
            $this->icon = apply_filters('woocommerce_utopiapay_icon', plugins_url('utopiapay.png', __FILE__));
            $this->has_fields = false;
            $this->method_title = __('UtopiaPay', 'utopiapay');
            $this->method_description = __('Accept payments in Crypton (CRP) and Utopia USD (UUSD) via UtopiaPay', 'utopiapay');
            $this->ipn_url   = add_query_arg('wc-api', 'WC_Gateway_UtopiaPay', home_url('/'));


            // Load the settings.
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->merchant_id = $this->get_option('merchant_id');
            $this->token = $this->get_option('token');
			// $this->site_url = $this->get_option('site_url');
			// $this->site_url_inside = $this->get_option('site_url_inside');
			$this->insideOutside = $this->get_option('insideOutside');
            $this->secret_key = $this->get_option('secret_key');

            $this->log = new WC_Logger();

            // Actions
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_receipt_utopiapay', array($this, 'receipt_page'));
            
            // thank you page
            add_action('woocommerce_thankyou_utopiapay', array($this, 'thankyou_page'));

            // Payment listener/API hook
            add_action('woocommerce_api_wc_gateway_utopiapay', array($this, 'check_utopiapay_response'));

            if (!$this->is_valid_for_use()) $this->enabled = false;
        }

        /**
         * Check if this gateway is enabled and available in the user's country
         *
         * @access public
         * @return bool
         */
        function is_valid_for_use()
        {
            // if (!in_array(get_woocommerce_currency(), apply_filters('woocommerce_utopiapay_supported_currencies', array('UUSD', 'CRP')))) return false;
            return true;
        }



       

        /**
         * Admin Panel Options
         * - Options
         *
         * @since 1.0.0
         */
        public function admin_options()
        {
?>
<h3><?php _e('UtopiaPay', 'utopiapay'); ?></h3>
<p><?php _e('Accept payments in Crypton (CRP) and Utopia USD (UUSD) via UtopiaPay', 'utopiapay'); ?></p>

<?php if ($this->is_valid_for_use()) : ?>

<table class="form-table">
    <?php $this->generate_settings_html(); ?>
</table>

<?php else : ?>
<div class="inline error">
    <p><strong><?php _e('Gateway Disabled', 'utopiapay'); ?></strong>:
        <?php _e('UtopiaPay does not support your store currency.', 'utopiapay'); ?></p>
</div>
<?php
            endif;
        }



        /**
         * Initialise Gateway Settings Form Fields
         *
         * @access public
         * @return void
         */
        function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'utopiapay'),
                    'type' => 'checkbox',
                    'label' => __('Enable UtopiaPay', 'utopiapay'),
                    'default' => 'yes'
                ),
                'title' => array(
                    'title' => __('Title', 'utopiapay'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'utopiapay'),
                    'default' => __('UtopiaPay', 'utopiapay'),
                    'desc_tip' => true,
                ),
                'description' => array(
                    'title' => __('Description', 'utopiapay'),
                    'type' => 'textarea',
                    'description' => __('This controls the description which the user sees during checkout.', 'utopiapay'),
                    'default' => __('Pay in Crypton (CRP) or Utopia USD (UUSD) via UtopiaPay', 'utopiapay'),
                    'desc_tip' => true,
                ),
                'merchant_id' => array(
                    'title' => __('UtopiaPay ID', 'utopiapay'),
                    'type' => 'text',
                    'description' => __('Please enter your UtopiaPay ID.', 'utopiapay'),
                    'default' => '',
                    'desc_tip' => true,
                ),
                'token' => array(
                    'title' => __(' UtopiaPay Token', 'utopiapay'),
                    'type' => 'text',
                    'description' => __('Please enter your UtopiaPay Token.', 'utopiapay'),
                    'default' => '',
                    'desc_tip' => true,
                ),
                'insideOutside' => array(
                    'title' => __('Inside or Outside Utopia', 'utopiapay'),
                    'type' => 'select',
                    'description' => __('Select if your site inside or outside Utopia.', 'utopiapay'),
                    'default' => 'inside',
                    'options' => array(
                        'inside' => __('Inside Utopia', 'utopiapay'),
                    'outside' => __('Outside Utopia', 'utopiapay'),
                    'auto' => __('Auto-detect (not secure and can fail)', 'utopiapay'),
                    ),
                    'desc_tip' => true,
                ),
                'secret_key' => array(
                    'title' => __('UtopiaPay Secret Key', 'utopiapay'),
                    'type' => 'text',
                    'description' => __('Please enter your UtopiaPay Secret Key.', 'utopiapay'),
                    'default' => '',
                    'desc_tip' => true,
                ),

            );
        }


     

        // the payment gateway works making an post request with the data generated and and encrypting all
        // the data with the public key of the merchant.

        public function process_payment($order_id)
        {
            global $woocommerce;

            $order = new WC_Order($order_id);

            try {
                // get the coinfig data
                $token = $this->token;
                $id = $this->merchant_id;
                $order_id = $order->get_order_number();
                $destinationPageAfterPaymentSuccessfull = $this->get_return_url($order);
                $cancelUrl = esc_url_raw($order->get_cancel_order_url_raw());
                $currency = $order->get_order_currency();
                $secretKey = $this->secret_key;
                $amount = $order->get_total();

                

                // WordPress get url 
                $url = get_site_url();

                // check if the order is inside or outside utopia
                if ($this->insideOutside == 'inside') {
                    $isInside = "true";
                    $uUrl = "http://utopiapay";
                } else if ($this->insideOutside == 'outside') {
                    $isInside = "false";
                    $uUrl = "https://utopiapay.io";
                }else if ($this->insideOutside == 'auto') {
                    // check if the host url from server host is http or https (inside or outside)
                    // we use $_SERVER['HTTPS'] instead of $url due user can host the same site 
                    // with same url inside utopia
                    if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on'){   
                        $isInside = "false";
                        $uUrl = "https://utopiapay.io";  
                    }else  {
                        $isInside = "true";
                        $uUrl = "http://utopiapay"; 
                    } 
                }
                

                // generate the data to send to UtopiaPay Gateway
                $data = array(
                    "amount" => $amount,
                    "orderID" => $order_id,
                    "token" => $token,
                    "webTo" => $destinationPageAfterPaymentSuccessfull,
                    "webCancel" => $cancelUrl,
                    "siteUrl" => $url,
                    "ipnUrl" => $url . "/?wc-api=WC_Gateway_UtopiaPay",
                    "isInside" => $isInside,
                    "id" => $id,
                    "currency" => $currency,
                );
                $json = json_encode($data,JSON_FORCE_OBJECT,1024);

                $dataToCrypt = $json;
                $dataToCrypt = base64_encode($dataToCrypt);
                $nonce = random_bytes(SODIUM_CRYPTO_BOX_NONCEBYTES);
                    //wp_die(var_dump(($json)));


                $encryptedText = sodium_crypto_box($dataToCrypt, $nonce, base64_decode($secretKey));
                
                $dataCrypted = base64_encode($encryptedText);
                $nonce = base64_encode($nonce);
                
            } catch (Exception $e) {
                echo $e->getMessage();
            }

            // redirect to utopiapay gateway
            $data = base64_encode(json_encode([
                "id" => $id,
                "data" => $dataCrypted,
                "nonce" => $nonce
            ]));

            $url = $uUrl . "/checkout/pre/wp/" . $data;
            // add processing status
            $order->update_status('processing', __('Customer is being redirected to UtopiaPay...', 'utopiapay'));
            $woocommerce->cart->empty_cart();

            return array(
                'result' => 'success',
                'redirect' => $url
            );
        }

        // the function "process_payment" will be called when the user will click on the "place order" button
        public function receipt_page($order)
        {
            echo '<p>' . __('Thank you for your order, please click the button below to pay with UtopiaPay.', 'utopiapay') . '</p>';
            echo $this->generate_utopiapay_form($order);
        }

        // thank you
        function thankyou_page()
        {
            echo '<p>' . __('Thank you for your order. This order was paid via <a href="https://utopiapay.io">UtopiaPay</a> ', 'utopiapay') . '</p>';
        }


        // function to get the get data returned from the payment gateway: status, paymentID, name


        function successful_request($posted)
        {
            global $woocommerce;

            $order = false;
            $error_msg = "Unknown error";
            $auth_ok = false;



            if (isset($posted['status']) && $posted['status'] == "cancel") {
                
                $order = wc_get_order($posted['orderID']);
                $order->update_status('cancelled', __('Order cancelled', 'utopiapay'));
                // $woocommerce->cart->empty_cart();
                exit;
            }

            if (isset($posted['status']) && $posted['status'] == "paying") {
                $order = wc_get_order($posted['orderID']);
                // add pending payment status
                $order->update_status('pending', __('Order is being processed by UtopiaPay', 'utopiapay'));
                // $woocommerce->cart->empty_cart();
                exit;
            }
            
            if(isset($posted['status']) && $posted['status'] == "illegal"){
                $order = wc_get_order($posted['orderID']);
                $order->update_status('failed', __('Order failed due illegal action', 'utopiapay'));
                // $woocommerce->cart->empty_cart();
                exit;
            }

            $posted = stripslashes_deep($posted);

            // get data patymentID, name and status
            $data = $posted['data'];
            $data = base64_decode($data);
            $cipher = "AES-128-CTR";
            $iv = base64_decode($posted['iv']);

            // the token is the key
            $key = $this->token;
            $decrypted = openssl_decrypt($data, $cipher, $key, 0, $iv);
            $sadas = $decrypted;
            $decrypted = json_decode($decrypted);


            // get the derypted data from the gateway
            $paymentID = $decrypted->paymentID;
            $name = $decrypted->name;
            $status = $decrypted->status;
            $order_id = $decrypted->orderID;
            $token = $decrypted->token;
            $amount = $decrypted->amount;
            $currency = $decrypted->currency;
            $order = wc_get_order($order_id);
            
            // add note to the order
            $order->add_order_note(__('Checking UtopiaPay payment', 'utopiapay'));

            // check if the order is correct (multiple check in case fails, sometimes happens)
            for($i = 0; $i < 5; $i++){
                if ($order === FALSE) {
                    sleep(0.5);
                    $order = wc_get_order($order_id);
                } else {
                    break;
                }
            }
            if ($order === FALSE) {
                // $order->add_order_note(__('UtopiaPay payment checking failed. The payment may be completed.', 'utopiapay'));
                // update status to on hold
                $order->update_status('on-hold', __('UtopiaPay payment checking failed. The payment may be completed.', 'utopiapay'));
                exit;
            }

            // check if all is set
            if (isset($paymentID) && isset($name) && isset($status)) {
                // check if payment is successful



                $order->add_order_note('UtopiaPay Payment Status: ' . $status);
                $woocommerce->cart->empty_cart();
                if ($status == "success") {
                    
                    update_post_meta($order->get_id(), 'UtopiaPay payment complete', 'Yes');
                    $order->update_status('completed', __('Order completed', 'utopiapay'));
                    $order->payment_complete();

                    // Add note about how much user payed and the currency
                    $order->add_order_note(__('Amount: ' . $amount . ' ' . $currency.' - Payment ID: '.$paymentID, 'utopiapay'));

                    exit;
                } else {
                    $order->update_status('failed', __('Payment failed', 'utopiapay'));
                    exit;
                }
            } else if (isset($status)) {
                if ($status == "cancel") {
                    $order->update_status('cancelled', __('Payment cancelled', 'utopiapay'));
                    exit;
                } else {
                    $order->update_status('failed', __('Payment failed', 'utopiapay'));
                    exit;
                }
            } else {
                $order->update_status('failed', __('Payment failed', 'utopiapay'));
                exit;
            }
        }



        /**
		 * Check for UtopiaPay IPN Response
		 *
		 * @access public
		 * @return void
		 */

        function check_utopiapay_response()
        {
            // global $woocommerce;

			@ob_clean();

            if (!empty($_POST)) {

                $this->successful_request($_POST);
            } else {
                wp_die("UtoiaPay Request Error");
            }
        }
    }

    class WC_UtopiaPay extends WC_Gateway_Utopiapay
    {
        public function __construct()
        {
            _deprecated_function('WC_UtopiaPay', '1.4', 'WC_Gateway_Utopiapay');
            parent::__construct();
        }
    }
}